package cn.com.mma.mobile.tracking.api;

import java.util.Timer;
import java.util.TimerTask;
import android.content.Context;
import android.content.SharedPreferences;
import cn.com.mma.mobile.tracking.util.Logger;
import cn.com.mma.mobile.tracking.util.SdkConfigUpdateUtil;
import cn.com.mma.mobile.tracking.util.SharedPreferencedUtil;

/**
 * MMAChinaSDK API入口类
 */
public class Countly {


    private SendMessageThread sendNormalMessageThread = null;
    private SendMessageThread sendFailedMessageThread = null;

    private Timer normalTimer = null;
    private Timer failedTimer = null;


    private volatile boolean sIsInitialized = false;
    private Context mContext;
    private RecordEventMessage mUrildBuilder;

    private static final String EVENT_CLICK = "onClick";
    private static final String EVENT_EXPOSE = "onExpose";

    private static Countly mInstance = null;


    public static Countly sharedInstance() {
        if (mInstance == null) {
            synchronized (Countly.class) {
                if (mInstance == null) {
                    mInstance = new Countly();
                }
            }
        }
        return mInstance;
    }


    /**
     * 设置是否打开log输出
     */
    public void setLogState(boolean isPrintOut) {
        Logger.DEBUG_LOG = isPrintOut;
    }

    /**
     * 初始化SDK
     */
    public void init(Context context, String configURL) {
        if (context == null) {
            Logger.e("Countly.init(...) failed:Context can`t be null");
            return;
        }
        if (sIsInitialized) {
            return;
        } else {
            sIsInitialized = true;
        }

        try {
            Context appContext = context.getApplicationContext();
            mContext = appContext;
            normalTimer = new Timer();
            failedTimer = new Timer();
            mUrildBuilder = RecordEventMessage.getInstance(context);

            SdkConfigUpdateUtil.initSdkConfigResult(context, configURL);
            startTask();

        } catch (Exception e) {
            Logger.e("Countly init failed");
        }
    }


    private void startTask() {
        try {
            normalTimer.schedule(new TimerTask() {
                public void run() {
                    startNormalRun();
                }
            }, 0, Constant.ONLINECACHE_QUEUEEXPIRATIONSECS);

            failedTimer.schedule(new TimerTask() {
                public void run() {
                    startFailedRun();
                }
            }, 0, Constant.OFFLINECACHE_QUEUEEXPIRATIONSECS * 1000);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void startNormalRun() {
        try {
            if (sendNormalMessageThread != null && sendNormalMessageThread.isAlive()) return;

            SharedPreferences sp = SharedPreferencedUtil.getSharedPreferences(mContext, SharedPreferencedUtil.SP_NAME_NORMAL);
            if ((sp == null) || (sp.getAll().isEmpty())) return;

            sendNormalMessageThread = new SendMessageThread(SharedPreferencedUtil.SP_NAME_NORMAL, mContext, true);
            sendNormalMessageThread.start();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void startFailedRun() {
        try {
            if (sendFailedMessageThread != null && sendFailedMessageThread.isAlive()) return;

            SharedPreferences sp = SharedPreferencedUtil.getSharedPreferences(mContext, SharedPreferencedUtil.SP_NAME_FAILED);
            if ((sp == null) || (sp.getAll().isEmpty())) return;

            sendFailedMessageThread = new SendMessageThread(SharedPreferencedUtil.SP_NAME_FAILED, mContext, false);
            sendFailedMessageThread.start();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 点击事件监测
     * @param adURL 广告点击监测链接
     */
    public void onClick(String adURL) {

        triggerEvent(EVENT_CLICK, adURL);
    }


    /**
     * 曝光事件监测
     * @param adURL 广告曝光监测链接
     */
    public void onExpose(String adURL) {

        triggerEvent(EVENT_EXPOSE, adURL);
    }


    private void triggerEvent(String eventName, String adURL) {

        if (sIsInitialized == false || mUrildBuilder == null) {
            Logger.e("The static method " + eventName + "(...) should be called before calling Countly.init(...)");
            return;
        }

        switch (eventName) {
            case EVENT_CLICK:
            case EVENT_EXPOSE:
                if (sIsInitialized && mUrildBuilder != null) {
                    mUrildBuilder.recordEventWithUrl(adURL);
                }
                break;
        }
    }


    public void terminateSDK() {
        try {
            if (normalTimer != null) {
                normalTimer.cancel();
                normalTimer.purge();
            }
            if (failedTimer == null) {
                failedTimer.cancel();
                failedTimer.purge();
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            normalTimer = null;
            failedTimer = null;
            sendNormalMessageThread = null;
            sendFailedMessageThread = null;
            mUrildBuilder = null;
        }
    }


}
